@@ -84,7 +84,7 @@ sub log_err {
 
 sub is_subset {
   return 0 if grep /^-no-subset$/, @ARGV;
-  return ! -d '.svn' || grep /^-subset$/, @ARGV;
+  return ! (-d '.svn' or -d'.git') || grep /^-subset$/, @ARGV;
 }
 
 sub get_module_list {
@@ -190,7 +190,10 @@ sub testcc   {
     my $cwd = Cwd::getcwd();
     # posix shell only, but we are using a posix shell here. XXX -Wb=-uTest::Builder
     my $X = $^X =~ m/\s/ ? qq{"$^X"} : $^X;
-    $self->prefs->{test}->{commandline} = "for t in t/*.t; do echo \"# \$t\"; $X -Iblib/arch -Iblib/lib -I\"$cwd/blib/arch\" -I\"$cwd/blib/lib\" \"$cwd/blib/script/perlcc\" -T -r \$t; done";
+    $self->prefs->{test}->{commandline} =
+      "for t in t/*.t; do "
+	. "echo \"# \$t\"; $X -Iblib/arch -Iblib/lib -I\"$cwd/blib/arch\" -I\"$cwd/blib/lib\" \"$cwd/blib/script/perlcc\" -T -r \$t;"
+     ."done";
     $self->prefs->{test_report} = ''; # XXX ignored!
     $self->{make_test} = 'NO'; # override YAML check "Has already been tested successfully"
     $self->test(@_);
